/* file_byte_stream.cpp
 *
 * (C) 2000-2001 empeg ltd, http://www.empeg.com
 *
 * This software is licensed under the GNU General Public Licence (see file
 * COPYING), unless you possess an alternative written licence from empeg ltd.
 *
 * $Header: /home/cvs/symo/empeg/file_byte_stream.cpp,v 1.1 2001/08/24 20:03:51 peter Exp $
 */

#include <errno.h>
#include "empeg_error.h"
#include "file_byte_stream.h"

ReadFileStream::ReadFileStream()
    : m_f(NULL)
{
}

ReadFileStream::~ReadFileStream()
{
    Close();
}

STATUS ReadFileStream::Open(const char *file)
{
    m_f = fopen(file, "rb");
    if (m_f == NULL)
	return MakeErrnoStatus(errno);
    
    return S_OK;
}

void ReadFileStream::Close()
{
    if (m_f)
    {
	fclose(m_f);
	m_f = NULL;
    }
}

STATUS ReadFileStream::Read(void *buffer, unsigned bytesToRead, unsigned *pBytesRead)
{
    size_t rc = fread(buffer, 1, bytesToRead, m_f);

    if (rc < bytesToRead && ferror(m_f))
	return MakeErrnoStatus(errno);

    *pBytesRead = rc;

    if (feof(m_f))
	return S_FALSE;

    return S_OK;
}

STATUS ReadFileStream::Write(const void * /*buffer*/, unsigned /*bytesToWrite*/, unsigned * /*pBytesWritten*/)
{
    return E_NOTIMPL;
}

STATUS ReadFileStream::Tell(unsigned *position)
{
    long pos = ftell(m_f);
    if (pos < 0)
	return MakeErrnoStatus(errno);

    *position = pos;

    return S_OK;
}

STATUS ReadFileStream::SeekAbsolute(unsigned absolutePosition)
{
    if (fseek(m_f, absolutePosition, SEEK_SET) < 0)
	return MakeErrnoStatus(errno);
    
    return S_OK;
}

STATUS ReadFileStream::SeekRelative(signed int relativePosition)
{
    if (fseek(m_f, relativePosition, SEEK_CUR) < 0)
	return MakeErrnoStatus(errno);
    
    return S_OK;
}

#if defined(WIN32)
#define empeg_stat _stat
#define empeg_fstat _fstat
#define empeg_fileno _fileno
#else
#define empeg_stat stat
#define empeg_fstat fstat
#define empeg_fileno fileno
#endif

STATUS ReadFileStream::Length(unsigned *length)
{
    struct empeg_stat st;
    if (empeg_fstat(empeg_fileno(m_f), &st) != 0)
    {
	Close();
	return MakeErrnoStatus(errno);
    }
    
    *length = st.st_size;
    
    return S_OK;
}
