/* mp3_frame_info.cpp
 *
 * (C) 2000 empeg ltd, http://www.empeg.com
 *
 * This software is licensed under the GNU General Public Licence (see file
 * COPYING), unless you possess an alternative written licence from empeg ltd.
 *
 * $Header: /home/cvs/symo/empeg/mp3_frame_info.cpp,v 1.2 2001/10/01 18:46:45 peter Exp $
 */

#include "mp3_frame_info.h"
#include "extract_tags.h"
#include <stdio.h>
#include "stream.h"

/* ARM decoder doesn't cope with layers 1 and 2 (codes 3 and 2) -- remove this
 * hack once it's fixed.
 */
#define MPEG_LAYER3_ONLY 0

STATUS FrameInfoExtract::Extract(int dataOffset, ExtractTagsObserver *pObserver)
{
    /* pdh 18-Sep-00: changed to check for CBR first, as only the CBR check
     * searches for the header (i.e. skips ID3v2).
     */
    m_seekPos = 0;
    m_length = 0;
    STATUS rc = m_stream->Length(&m_length);
    if (FAILED(rc))
	return rc;

    rc = ParseMP3Frames(pObserver);
    if (FAILED(rc))
        return rc;

    m_seekPos = m_firstHeader;

    // Try for a VBR header...
    if (GetVBRHeader())
	ParseVBRHeader(pObserver);

    return S_OK;
}

// get Xing header data    
bool FrameInfoExtract::GetVBRHeader()
{
    // get selected MPEG header data
    if ( GetByte() < 0 )   // skip buf[0]
	return false;

    int h_id       = (GetByte()>>3)&1;
    int h_sr_index = (GetByte()>>2)&3;
    int h_mode     = (GetByte()>>6)&3;

    m_seekPos += FigureVBRHeaderOffset(h_id, h_mode);

    if (GetByte() != 'X') return false;    // fail
    if (GetByte() != 'i') return false;    // header not found
    if (GetByte() != 'n') return false;
    if (GetByte() != 'g') return false;

    m_vbrHeader.h_id = h_id;

    static int sr_table[4] = { 44100, 48000, 32000, 99999 };
    m_vbrHeader.samprate = sr_table[h_sr_index];

    if (h_id == 0)
	m_vbrHeader.samprate >>= 1;

    // get flags
    int head_flags = GetWord();
    m_vbrHeader.flags = head_flags;

    if (head_flags & FRAMES_FLAG)
	m_vbrHeader.frames = GetWord();

    if (head_flags & BYTES_FLAG)
	m_vbrHeader.bytes = GetWord();

    if (head_flags & TOC_FLAG)
    {
	for(int i=0; i<100; i++)
	    m_vbrHeader.toc[i] = GetByte();
    }

    m_vbrHeader.vbr_scale = 1;
    if (head_flags & VBR_SCALE_FLAG)
	m_vbrHeader.vbr_scale = GetWord();

#ifdef PRINT_VBR_TOC
    for(int i = 0; i < 100; i++)
    {
	if ((i % 10) == 0)
	    printf("\n");
	printf(" %3d", m_vbrHeader.toc[i]);
    }
#endif

    return true;       // success
}

int FrameInfoExtract::FigureVBRHeaderOffset(int h_id, int h_mode)
{
    if (h_id)	// mpeg1
    {        
	if (h_mode != 3)
	    return 32;
	else
	    return 17;
    }
    else	// mpeg2
    {      
	if (h_mode != 3)
	    return 17;
	else
	    return 9;
    }
}

bool FrameInfoExtract::ParseVBRHeader(ExtractTagsObserver *pObserver)
{
    // Try and get mp3 header to work out stereo status
    m_mp3Header.channelmode_code = 0;
    m_mp3Header.bitrate = 0;
    GetMP3Sync(4);
    
    // No divide by 0's!
    int length_ms;
    if ((m_vbrHeader.samprate / 1000) != 0)
    {
	length_ms = (m_vbrHeader.frames * 1152) / (m_vbrHeader.samprate / 1000);
    }
    else
    {
	length_ms = 0;
    }
    
    char bitrate[32];
    FigureAverageBitrate(length_ms, bitrate, 32);
    
    pObserver->OnExtractTag("duration", length_ms);
    pObserver->OnExtractTag("bitrate", bitrate);
    pObserver->OnExtractTag("samplerate", m_vbrHeader.samprate);

    return true;
}

#ifdef _MSC_VER
#define empeg_snprintf _snprintf
#else
#define empeg_snprintf snprintf
#endif

void FrameInfoExtract::FigureAverageBitrate(int length_ms, char *buffer, int bufflen)
{
    // Note the average bitrate
    if ((length_ms / 1000) != 0)
    {
	empeg_snprintf(buffer, bufflen, "v%c%ld",
	    (m_mp3Header.channelmode_code == 3) ? 'm' : 's',
	    (m_vbrHeader.bytes * 8) / length_ms);
    }
    else
    {
	// Unknown bitrate: use bitrate of first header
	empeg_snprintf(buffer, bufflen, "v%c%d",
	    (m_mp3Header.channelmode_code == 3) ? 'm' : 's',
	    m_mp3Header.bitrate / 1000);
    }
    buffer[31] = '\0';
}

STATUS FrameInfoExtract::ParseMP3Frames(ExtractTagsObserver *pObserver)
{
    // It's a standard MP3 file.  Look for sync in the first 192Kb (3 lots of 64).
    for (int tries = 3; tries; tries--)
    {
	if (GetMP3Sync(65536))
	{
	    // We got sync
	    int first_sample_rate = m_mp3Header.samplerate;
	    int first_header = m_seekPos - 4;

	    // To be valid, the next frame should be the right distance away.
	    Skip(m_mp3Header.length - 4);
	    if (GetMP3Sync(4))
	    {
		// Looks good so far.  Do the sample rates match?
		if (first_sample_rate == m_mp3Header.samplerate)
		{
		    // They do.  There's a good chance that this is valid data.
		    // Note down the bitrate and sample rate.
		    char br[32];
		    sprintf(br, "f%c%d",  (m_mp3Header.channelmode_code == 3) ? 'm' : 's', m_mp3Header.bitrate / 1000);
		    pObserver->OnExtractTag("bitrate", br);
		    pObserver->OnExtractTag("samplerate", m_mp3Header.samplerate);

		    // As this is the first frame, this is really where we
		    // should be streaming from
		    m_firstHeader = first_header;
		    pObserver->OnExtractTag("offset", first_header);
		    m_seekPos = first_header;

		    // Work out length
		    int length_ms = 0;
		    if ((m_mp3Header.bitrate / 1000) != 0)
		    {
			length_ms = ((m_length - first_header) * 8) / (m_mp3Header.bitrate / 1000);
		    }

		    pObserver->OnExtractTag("duration", length_ms);
		    
		    return S_OK;
		}
	    }
	}
	else
	{
	    // Back up a bit, just in case we managed to straddle the sync...
	    Skip(-4);
	}
    }

    return E_INVALID_MP3;
}

bool FrameInfoExtract::GetMP3Sync(int distance)
{
    // Search until end of file or distance bytes
    if ((m_seekPos + distance) > m_length)
	distance = (m_length - m_seekPos);

    distance += m_seekPos;

    // Nothing in the shift register yet
    m_shiftReg = 0;

    // On an empty drive, you can seek forever...
    while((int) m_seekPos < distance)
    {
	int byte = GetByte();
	
	if ( byte < 0 )
	    return false;

	m_shiftReg = (m_shiftReg << 8) | byte;
	
	// Got sync yet?
	if ((m_shiftReg & 0xffe00000) == 0xffe00000)
	{
	    if (GetMP3Header() > 0)
	        return true;
	}
    }

    return false;
}	

void FrameInfoExtract::Skip(int howfar)
{
    if ((m_seekPos + howfar) < m_length)
    {
	m_seekPos += howfar;

	// The shift register will be junk
	m_shiftReg = 0;
    }
}

int FrameInfoExtract::GetByte(void)
{
    // Fill buffer as necessary
    if (((int) m_seekPos < m_buffer_start) || ((int) m_seekPos - m_buffer_start) >= m_buffer_length)
    {
	// Valid?
	if (m_seekPos >= m_length)
	{
	    // Simulate reading zeros beyond the end
	    m_seekPos++;
	    return 0;
	}

	// Refill it
	m_buffer_start = m_seekPos;
	if (FAILED(m_stream->SeekAbsolute(m_seekPos)))
	    return -1;

	STATUS rc = m_stream->Read(m_buffer, sizeof(m_buffer), 
				   (unsigned int*)&m_buffer_length);
	if (FAILED(rc))
	    return -1;
    }

    return(m_buffer[(m_seekPos++ - m_buffer_start)]);
}


int FrameInfoExtract::GetMP3Header()
{
    m_mp3Header.layer_code = (m_shiftReg & 0x60000) >> 17;
    
    // Invalid layer?
    if (m_mp3Header.layer_code == 0)
	return 0;

#if MPEG_LAYER3_ONLY
    /* ARM decoder doesn't cope with layers 1 and 2 (codes 3 and 2) -- remove 
     * this hack once it's fixed.
     */
    if (m_mp3Header.layer_code != 1)
        return 0;
#endif

    m_mp3Header.mpeg_version_code = (m_shiftReg & 0x180000) >> 19;
    
    // Reserved version code
    if (m_mp3Header.mpeg_version_code == 1) 
	return 0;

    m_mp3Header.bitrate_code = (m_shiftReg & 0xf000) >> 12;

    // We don't do free format or invalid bitrates
    if (m_mp3Header.bitrate_code == 0 || m_mp3Header.bitrate_code == 0xf)
	return 0;
    
    m_mp3Header.samplerate_code = (m_shiftReg & 0xc00) >> 10;

    // We don't do bad sample rates
    if (m_mp3Header.samplerate_code == 0x3)
	return 0;
    
    /* 0=stereo, 1=js, 2=dual channel, 3=mono */
    m_mp3Header.channelmode_code = (m_shiftReg & 0xc0) >> 6;

    m_mp3Header.layer_number = 4 - m_mp3Header.layer_code;
    m_mp3Header.padded = (m_shiftReg & 0x200) >> 9;

    m_mp3Header.bitrate = FigureMP3Bitrate(m_mp3Header.mpeg_version_code, m_mp3Header.layer_number, m_mp3Header.bitrate_code);
    m_mp3Header.samplerate = FigureMP3Samplerate(m_mp3Header.mpeg_version_code, m_mp3Header.samplerate_code);

    if (m_mp3Header.layer_number == 1)
    {
	m_mp3Header.length = (12 * m_mp3Header.bitrate) / m_mp3Header.samplerate + m_mp3Header.padded * 4;
    }
    else
    {
	// This fix is based on empirical evidence & not anything concrete
	// that I could find!
	if (m_mp3Header.mpeg_version_code==2) {
	    m_mp3Header.length = (72 * m_mp3Header.bitrate) / m_mp3Header.samplerate + m_mp3Header.padded;
	} else {
	    m_mp3Header.length = (144 * m_mp3Header.bitrate) / m_mp3Header.samplerate + m_mp3Header.padded;
	}
    }

    return m_mp3Header.length;
}

int FrameInfoExtract::FigureMP3Bitrate(int mpeg_version_code, int layer_number, int bitrate_code)
{
    static const short bitrate_table[2][3][16] =
    {
	// MPEG2 & 2.5
	{
	    // Layer 1
	    {
		0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, -1
	    },
	    // Layer 2
	    {
		0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1
	    },
	    // Layer 3
	    {
		0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160, -1
	    }
	},
	// MPEG1
	{
	    // Layer 1
	    {
		0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, -1
	    },
	    // Layer 2
	    {
		0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384, -1
	    },
	    // Layer 3
	    {
		0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, -1
	    }
	}
    };

    return 1000 * bitrate_table[mpeg_version_code & 1]
				[layer_number - 1]
				[bitrate_code];
}

int FrameInfoExtract::FigureMP3Samplerate(int mpeg_version_code, int samplerate_code)
{
    static const int samplerate_table[4][4] =
    {
        // MPEG Version 2.5
        {
	    11025, 12000, 8000, -1
        },
        // Reserved
        {
	    -1, -1, -1, -1
        },
        // MPEG Version 2
        {
	    22050, 24000, 16000, -1
        },
        // MPEG Version 1
        {
	    44100, 48000, 32000, -1
        }
    };

    return samplerate_table[mpeg_version_code]
			    [samplerate_code];
}

unsigned long FrameInfoExtract::GetWord()
{
    unsigned int w;

    w  = GetByte() << 24;
    w |= GetByte() << 16;
    w |= GetByte() << 8;
    w |= GetByte();
    
    return w;
}
