/* genre.cpp
 *
 * List of ID3 genres
 *
 * (C) 2000 empeg ltd, http://www.empeg.com
 *
 * This software is licensed under the GNU General Public Licence (see file
 * COPYING), unless you possess an alternative written licence from empeg ltd.
 */

#include <algorithm>

#include "id3genres.h"

const char *const ID3Genres::m_genres[] = {
    "Blues",		// 0
    "Classic Rock",
    "Country",
    "Dance",
    "Disco",
    "Funk",
    "Grunge",
    "Hip-Hop",
    "Jazz",
    "Metal",

    "New Age",		// 10
    "Oldies",
    "Other",
    "Pop",
    "R&B",
    "Rap",
    "Reggae",
    "Rock",
    "Techno",
    "Industrial",

    "Alternative",	// 20
    "Ska",
    "Death Metal",
    "Pranks",
    "Soundtrack",
    "Euro-Techno",
    "Ambient",
    "Trip-Hop",
    "Vocal",
    "Jazz+Funk",

    "Fusion",		// 30
    "Trance",
    "Classical",
    "Instrumental",
    "Acid",
    "House",
    "Game",
    "Sound Clip",
    "Gospel",
    "Noise",

    "AlternRock",	// 40
    "Bass",
    "Soul",
    "Punk",
    "Space",
    "Meditative",
    "Instrumental Pop",
    "Instrumental Rock",
    "Ethnic",
    "Gothic",

    "Darkwave",		// 50
    "Techno-Industrial",
    "Electronic",
    "Pop-Folk",
    "Eurodance",
    "Dream",
    "Southern Rock",
    "Comedy",
    "Cult",
    "Gangsta",

    "Top 40",		// 60
    "Christian Rap",
    "Pop/Funk",
    "Jungle",
    "Native American",
    "Cabaret",
    "New Wave",
    "Psychadelic",
    "Rave",
    "Showtunes",

    "Trailer",		// 70
    "Lo-Fi",
    "Tribal",
    "Acid Punk",
    "Acid Jazz",
    "Polka",
    "Retro",
    "Musical",
    "Rock & Roll",
    "Hard Rock",

    "Folk",		// 80
    "Folk-Rock",
    "National Folk",
    "Swing",
    "Fast Fusion",
    "Bebob",
    "Latin",
    "Revival",
    "Celtic",
    "Bluegrass",

    "Avantgarde",	// 90
    "Gothic Rock",
    "Progressive Rock",
    "Psychedelic Rock",
    "Symphonic Rock",
    "Slow Rock",
    "Big Band",
    "Chorus",
    "Easy Listening",
    "Acoustic",

    "Humour",		// 100
    "Speech",
    "Chanson",
    "Opera",
    "Chamber Music",

    "Sonata",
    "Symphony",
    "Booty Bass",
    "Primus",
    "Porn Groove",

    "Satire",		// 110
    "Slow Jam",
    "Club",
    "Tango",
    "Samba",

    "Folklore",
    "Ballad",
    "Power Ballad",
    "Rhythmic Soul",
    "Freestyle",

    "Duet",		// 120
    "Punk Rock",
    "Drum Solo",
    "Acapella",
    "Euro-House",

    "Dance Hall"
    "Goa",		// 126
    "Drum & Bass",
    "Club-House",
    "Hardcore",

    "Terror",		// 130
    "Indie",
    "Britpop",
    "Negerpunk",
    "Polsk Punk",

    "Beat",
    "Christian Gangsta",
    "Heavy Metal",
    "Black Metal",
    "Crossover",

    "Contemporary C",	// 140
    "Christian Rock",
    "Merengue",
    "Salsa",
    "Thrash Metal",
    "Anime",
    "JPop",
    "Synth Pop",	// 147

    "Sweetcorn Dub"     // 148
};

ID3Genres::ID3Genres()
{
    for ( unsigned int i=0; i<sizeof(m_genres)/sizeof(m_genres[0]); ++i )
    {
	std::string s = m_genres[i];
	std::transform(s.begin(), s.end(), s.begin(), tolower);
	m_map[s] = i;
    }
}

const char *ID3Genres::FromNumber(int i)
{
    if (i < 0 || i >= (sizeof(m_genres)/sizeof(m_genres[0])))
	return "";
    else
	return m_genres[i];
}

int ID3Genres::FromString(std::string s)
{
    std::transform(s.begin(), s.end(), s.begin(), tolower);

    map_t::iterator it = m_map.find(s);
    if ( it == m_map.end() )
	return -1;

    return it->second;
}
