/* symo.h
 *
 * Portable MP3 editing
 * (K) All Rites Reversed -- Copy What You Like (see file COPYING)
 *
 * Authors:
 *     Peter Hartley      <pdh@chaos.org.uk>
 */

#ifndef included_symo_h
#define included_symo_h

#include <string>
#include <vector>

#ifndef included_id3genres_h
#include "id3genres.h"
#endif

class SymoObserver
{
 public:
    // return true to cancel
    virtual bool onActivity( size_t sofar, size_t altogether ) = 0;

    // doesn't abort the operation
    virtual void onError( std::string ) = 0;

    // the numbered entry has changed (eg due to write)
    virtual void onEntryChanged( unsigned int ) = 0;
};

class Symo
{
 public:
    enum TagStatus {
	PLAIN,
	SUSPICIOUS,
	EDITED
    };

    struct Entry
    {
	std::string originalFilename; // before any renaming
	std::string filename;
	std::string title;
	std::string artist;
	std::string album;
	std::string year;
	std::string genre;
	std::string trackno;
	std::string bitrate;
	size_t sz;
	size_t sec;
	time_t mtime;
	bool edited : 1;
	bool deleted : 1;
	TagStatus filenameStatus, titleStatus, artistStatus, albumStatus,
	    yearStatus, genreStatus, tracknoStatus;
    };

    Symo();
    ~Symo();

    void addFile( const char *fname );
    unsigned int entries() const { return m_vec.size(); }
    const Entry& entry( unsigned int i ) const { return m_vec[i]; }
    void changeEntry( unsigned int i, const Entry& );
    void autoFixup( unsigned int i );
    void deleteFile( unsigned int i );
    void play( unsigned int i );

    enum Field {
	FILENAME,
	ARTIST,
	TITLE,
	ALBUM,
	YEAR,
        SIZE,
	MMSS
    };

    void sortEntries( Field f, bool ascending );

    unsigned int editedCount() const { return m_editedCount; }
    void read( SymoObserver* );
    void write( SymoObserver* );

 protected:
    typedef vector<Entry> vec_t;
    vector<Entry> m_vec;

    size_t m_editedCount;
    ID3Genres m_genres;
};

#endif
